; File......: ITOBASE.ASM
; Author....: Cyrus Patel
; Fido ID...: 2:243/7005.13
; Date......: $Date$
; Revision..: $Revision$
; Log file..: $Logfile$
;
; This is an original work by the seven dwarfs and is placed in the
; public domain.
;
; Modification history:
; ---------------------
;
; $Log$
;
;
;  $DOC$
;  $FUNCNAME$
;    ITOBASE()
;  $CATEGORY$
;    Conversion
;  $ONELINER$
;    Convert a numeric value to it's equivalent in a base between 2 and 36
;  $SYNTAX$
;    ITOBASE( <<nInterger>, <nBase> [, <nDigits>]> ) -> <cBasedString>
;  $ARGUMENTS$
;    <nInteger> is an unsigned integer numeric value to convert and
;    may be in the range 0 to 65535.
;    <nBase> is the base to which <nInteger> is to be converted.
;    <nDigits> is the number of digits the resulting string is to have.
;    By default, the result is left trimmed ie no leading zeros.
;    If the length of the result is greater than <nDigits>, the function
;    returns a string of '*'s. If the length of the result is less than
;    <nDigits>, the function pads the result with leading zeros. Passing
;    zero as an argument defaults to left trim.
;  $RETURNS$
;    ITOBASE() returns the value of <nInteger> as a character string
;    representing <nInteger> in the requested base.
;  $DESCRIPTION$
;    ITOBASE() is a low-level function that converts a decimal (base 10)
;    integer in the range 0 to 65535 to it's equivalent in any other
;    base in the range 2 to 36. ITOBASE() returns an empty string ("") for
;    invalid arguments. The complimentary function of ITOBASE() is BASETOI().
;
;    ITOBASE() is the functional equivalent of:
;    FUNCTION ITOBASE( nInteger, nBase, nLen )
;    LOCAL cInteger:=""
;    WHILE nInteger>0
;      cInteger:=SUBSTR( "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ",;
;                                  INT( nInteger%nBase )+1, 1 ) + cInteger
;      nInteger:=INT( nInteger/nBase )
;    END
;    IF !( EMPTY( nLen ) ) .and. LEN( cInteger )<>nLen
;      cInteger:=IF( LEN( cInteger ) < nLen, PADL( cInteger, nLen, "0" ),;
;                                    REPLICATE( "*", nLen ) )
;    END
;    RETURN( cInteger )
;  $EXAMPLES$
;     ITOBASE( 123, 2 )          -> "1110011"    //binary (left trimmed)
;     ITOBASE( 123, 2, 8 )       -> "01110011"   //binary (padded)
;     ITOBASE( 123, 2, 4 )       -> "****"       //overflow
;     ITOBASE( 1234, 8 )         -> "2322"       //octal
;     ITOBASE( 12345, 16 )       -> "3039"       //hexadecimal
;     ITOBASE( 123456 )          -> ""           //invalid argument error
;  $SEEALSO$
;     BASETOI()
;  $END$
;

PUBLIC       ITOBASE
EXTRN        __parNI:FAR
EXTRN        __parNL:FAR
EXTRN        __retC:FAR
DGROUP       GROUP  _DATA

_DATA        SEGMENT PUBLIC  'DATA'
;none: uses stack
_DATA        ENDS

_TEXT        SEGMENT WORD 'CODE'
             ASSUME cs:_TEXT, ds:DGROUP

ITOBASE      PROC    FAR

             push    bp              ; save the base pointer
             mov     bp,sp           ; start "new" stack

             mov     bx,2            ; parameter number 2
             push    bx              ; set it up
             call    __parNI         ; get the base
             add     sp,2            ; realign
             push    ax              ; save the radix
             mov     bx,3            ; parameter number 2
             push    bx              ; set it up
             call    __parNI         ; get the number of digits
             add     sp,2            ; realign
             push    ax              ; save the number of digits

             mov     bx,1            ; parameter number 1
             push    bx              ; set it up
             call    __parNL         ; get the integer in dx:ax
             add     sp,2            ; realign

             pop     bx              ; restore the number of digits
             pop     cx              ; restore radix into cx

             sub     sp,2            ; make space for terminating nulls
             mov     word ptr [bp-2],0 ; save 'em

            ;or      dx,dx           ; overflow?
            ;jnz     retResult       ; yes: error
             cmp     cx,2            ; invalid base?
             jb      retResult       ; yes: error
             cmp     cx,36           ; invalid base?
             ja      retresult       ; yes: error

convDigit:   div     cx              ; divide dx:ax by cx -> remainder in dx
             add     dx,'0'          ; convert to ASCII
             cmp     dx,'9'          ; is it '0'-'9'?
             jbe     saveDigit       ; if so, continue
             add     dx,'A'-'9'-1    ; put it in the range 'A'-'Z'

saveDigit:   xchg    dh,dl           ; put the digit in the upper byte
             push    dx              ; store it
             inc     sp              ; compensate for dl
             or      bx,bx           ; number of digits=0?
             jnz     length1         ; go see if it matches
             or      ax,ax           ; any more digits to process?
             jz      retResult       ; no: go return the result
extInteger:  cwd                     ; extend ax
             xor     dx,dx           ; clear sign
             jmp     convDigit       ; next digit

length1:     mov     dx,sp           ; make a copy
             add     dx,bx           ; number of digits
             add     dx,2            ; compensate for terminating nulls
             sub     dx,bp           ; difference is 0 if number of digits
             jnz     extInteger      ; no: more to do
             or      ax,ax           ; is ax already 0?
             jz      retResult       ; yes: the number fits
             mov     cx,bx           ; no: fill with '*'. 1. get the length
             mov     bx,sp           ; set the offset
length2:     mov     byte ptr ss:[bx],'*' ; put a '*'
             inc     bx              ; next position
             loop    length2         ; repeat for length=cx

retResult:   mov     bx,sp           ; point to the string
             push    ss              ; segment
             push    bx              ; offset
             call    __retC          ; send it back

             mov     sp,bp           ; realign
             pop     bp              ; clean up
             ret                     ; quit

ITOBASE      ENDP                    ; end of procedure
_TEXT        ENDS                    ; end of segment
             END                     ; end of assembly

<------------------------>

; File......: BASETOI.ASM
; Author....: Cyrus Patel
; Fido ID...: 2:243/7005.13
; Date......: $Date$
; Revision..: $Revision$
; Log file..: $Logfile$
;
; This is an original work by the seven dwarfs and is placed in the
; public domain.
;
; Modification history:
; ---------------------
;
; $Log$
;
;
;  $DOC$
;  $FUNCNAME$
;    BASETOI()
;  $CATEGORY$
;    Conversion
;  $ONELINER$
;    Convert an integer in a base from 2 to 36 to it's numeric equivalent
;  $SYNTAX$
;    BASETOI( <cBasedString>, <nBase> ) -> <nInteger>
;  $ARGUMENTS$
;    <cBasedString> is an integer value in base 2 to 32 in string form.
;    <nBase> is the base from which the string is to be converted.
;  $RETURNS$
;    <nInteger> is decimal numeric equivalent of <cBasedString>.
;  $DESCRIPTION$
;    BASETOI() is a low-level function that converts a string
;    representing an unsigned integer in a base 2 to 36, to it's
;    numeric base 10 form. BASETOI() returns -1 for an invalid argument
;    in <nBase>.
;
;    BASETOI() is the functional equivalent of:
;    FUNCTION BASETOI( cInteger, nBase )
;    LOCAL nInteger:=0, i
;    FOR i=1 TO LEN( cInteger )
;      nInteger:=( AT( SUBSTR( cInteger, i, 1 ),;
;         "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ" )-1 )+( nInteger*nBase )
;    END
;    RETURN( nInteger )
;  $EXAMPLES$
;    BASETOI( "01110011", 2 )     -> 123          //binary
;    BASETOI( "2322", 8 )         -> 1234         //octal
;    BASETOI( "3039", 16 )        -> 12345        //hexadecimal
;    BASETOI( "3OI0", 32 )        -> 123456       //theraininspainstaysmainl...
;  $SEEALSO$
;    ITOBASE()
;  $END$
;

PUBLIC       BASETOI
EXTRN        __parC:FAR
EXTRN        __parNI:FAR
EXTRN        __retNL:FAR
DGROUP       GROUP  _DATA

_DATA        SEGMENT PUBLIC  'DATA'
; none
_DATA        ENDS

_TEXT        SEGMENT PARA 'CODE'
             ASSUME cs:_TEXT, ds:DGROUP

BASETOI      PROC    FAR

             push    es              ; save registers
             push    di

             mov     bx,2            ; parameter number 2
             push    bx              ; set it up
             call    __parNI         ; get the base
             add     sp,2            ; realign

             push    ax              ; save the base
             mov     bx,1            ; parameter number 1
             push    bx              ; set it up
             call    __parC          ; get the string
             add     sp,2            ; realign

             mov     es,dx           ; set the read segment
             mov     di,ax           ; set the read offset

             xor     bx,bx           ; initialize work area
             xor     ax,ax           ; initialize result
             xor     dx,dx           ; initialize extension
             dec     ax              ; assume error
             dec     dx              ; extend sign

             pop     cx              ; base
             cmp     cx,2            ; base < 2
             jb      convDone        ; go return error
             cmp     cx,36           ; base > 36?
             ja      convDone        ; go return error
             inc     ax              ; make ax 0 again
             inc     dx              ; ditto

readDigit:   mov     bl,byte ptr es:[di] ; load character
             inc     di              ; point to next character
             cmp     bl,'9'          ; numeric?
             ja      convChar        ; no: do character
convNum:     cmp     bl,'0'          ; is character valid?
             jb      convDone        ; jump if below 0
             sub     bl,'0'          ; zero scale it
             jmp     convDigit       ; repeat
convChar:    or      bl,' '          ; make it lower case
             cmp     bl,'a'          ; is character valid?
             jb      convDone        ; jump if below 'a'
             cmp     bl,'z'          ; is character valid?
             ja      convDone        ; jump if below 'z'
             sub     bl,87           ; scale it
            ;jmp     convDigit       ; repeat

convDigit:   mul     cx              ; multiply the answer by the base
             add     ax,bx           ; add the digit
             jmp     readDigit       ; do the next digit

convDone:    push    dx              ; set up extension
             push    ax              ; set up integer
             call    __retNL         ; send it back
             add     sp,4            ; realign

             pop     di              ; restore the read segment
             pop     es              ; restore the read offset

             ret                     ; quit

BASETOI      ENDP                    ; end of procedure
_TEXT        ENDS                    ; end of segment
             END                     ; end of assembly

